<?php
/*
Plugin Name: Officers Directory
Plugin URI: http://www.douglasbell.us/plugins/
Description: Allows for the creation and management of a directory of officers for your organization, including separate listings of multiple types of officers. Includes an integrated contact form with reCAPTCHA support.
Version: 1.0.0
Author: Douglas Bell
Author URI: http://www.douglasbell.us
*/

/* Copyright 2010 Douglas Bell (email: douglas@douglasbell.us)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*--- PLUGIN INSTALLATION ---*/

// This will be run upon hitting the Officers Directory page in the admin

$officersDirVersion = '1.0.0';

function officersDirInstall()
{
	global $wpdb;
	$tableName = $wpdb->prefix . 'officers';
	
	// We don't already have our stuff installed, so let's do it now
	if ($wpdb->get_var("SHOW TABLES LIKE '$tableName'") != $tableName)
	{
		// Add the wp_officers table
		$sql = "CREATE TABLE IF NOT EXISTS {$wpdb->prefix}officers (
  positionShortname varchar(255) NOT NULL DEFAULT '',
  positionName varchar(255) NOT NULL DEFAULT '',
  positionOfficer varchar(255) NOT NULL DEFAULT '',
  positionCoOfficer varchar(255) NOT NULL,
  positionEmail varchar(255) NOT NULL,
  positionDescription text,
  positionType int(11) NOT NULL,
  positionOrder tinyint(3) NOT NULL,
  PRIMARY KEY (positionShortname)
)";
		$wpdb->query($sql);

		// Add the wp_officers_types table
		$sql = "CREATE TABLE IF NOT EXISTS {$wpdb->prefix}officers_types (
	positionTypeID int(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
	positionTypeName varchar(255) NOT NULL,
	positionTypeOrder tinyint(2) NOT NULL,
	PRIMARY KEY (positionTypeID)
)";
		$wpdb->query($sql);
		
		// Give us a default position type
		$wpdb->query("INSERT INTO {$wpdb->prefix}officers_types (positionTypeName, positionTypeOrder)
			VALUES ('Default', 1)");

		add_option('officersdir_version', $officersDirVersion); // No camelcase here for historical reasons
	}
}


/*--- PUBLIC-FACING SHORTCODES AND FORMS ---*/

// Include our friendly neighborhood files
include_once(dirname(__FILE__) . '/class.officerscontact.php');
include_once(dirname(__FILE__) . '/class.officersdir.php');
include_once(dirname(__FILE__) . '/class.officerstypes.php');

// Add shortcodes to WordPress for the display of the officers table & contact form
add_shortcode('officers-table', 'officersTableShortcode');
add_shortcode('officers-contact', 'contactFormShortcode');

/**
 * Displays the officers table using WordPress' Shortcode API
 */
function officersTableShortcode($atts, $content = null)
{
	// Get our shortcode attributes
	extract(shortcode_atts(array(
		'types' => '',
		'shortnames' => '',
		'descriptions' => 'false',
		'contactform' => '',
		'positiontitle' => 'Position',
		'officertitle' => 'Officer',
	), $atts));
	
	// If we have types or shortnames specified, explode them into an array
	$typesArray			= (!empty($types)) ? explode('|', $types) : array();
	$shortnamesArray	= (!empty($shortnames)) ? explode('|', $shortnames) : array();
	
	// If descriptions is the default, we'll assume it wasn't mentioned; if it was mentioned, assume they want them
	$showDescriptions	= ($descriptions == 'false') ? false : true;
	
	// Now initialize the class and get our table to display
	$officersDir = new OfficersDir();
	$officersDir->selectOfficers($typesArray, $shortnamesArray);
	return $officersDir->displayOfficersTable($contactform, $showDescriptions, $positiontitle, $officertitle);
}

/**
 * Displays the contact form using WordPress' Shortcode API
 * Processing the form is handled separately, but if the form is recognized as processed successfully,
 * display a success message in place of the contact form
 */
function contactFormShortcode($atts, $content = '')
{
	// Get our shortcode attributes
	extract(shortcode_atts(array(
		'types' => '',
		'shortnames' => ''
	), $atts));
	
	// If we have types or shortnames specified, explode them into an array
	$typesArray			= (!empty($types)) ? explode('|', $types) : array();
	$shortnamesArray	= (!empty($shortnames)) ? explode('|', $shortnames) : array();
	
	// If there was any content enclosed in the shortcode, we'll consider those to be our hidden fields
	$hiddenFields		= $content;
	
	// Globalize our Contact form class in case it already exists, but we'll also check to see that it does exist
	global $officersContact;
	if ($officersContact instanceof OfficersContact)
	{
		// Was the e-mail sent? If so, let's replace the form with a success message
		if ($officersContact->emailed)
		{
			return '<p style="text-align: center"><strong>Your e-mail was sent successfully. <a href="' . get_permalink() . '">Send another e-mail</a> or <a href="' . get_bloginfo('url') . '">return to the ' . get_bloginfo('name') . ' home page</a>.</strong></p>';
		}
	}
	// We haven't already processed the form, so let's instantiate the class
	else
	{
		$officersContact = new OfficersContact();
	}
	
	// Send our form to this same URL
	$action = get_permalink();
	
	// Regardless of submitted or not, we'll just do this to output the form
	return $officersContact->showContactForm($action, $hiddenFields, $typesArray, $shortnamesArray);
}

// If we find that the contact form was submitted, we want to process the form immediately
if (isset($_POST['officerContactForm_submit']))
{
	add_action('init', 'processContactForm');
}

/**
 * Processes the contact form by sending it along to our class to do all of the work
 */
function processContactForm()
{
	global $officersContact;
	
	// It probably doesn't already exist, but just in case...
	if (!($officersContact instanceof OfficersContact))
	{
		$officersContact = new OfficersContact();
	}
	
	$officersContact->processFormData($_POST['emailForm']);
	$officersContact->checkFormErrors();
	$officersContact->checkReCAPTCHAErrors($_POST['recaptcha_challenge_field'], $_POST['recaptcha_response_field']);
	if (!$officersContact->error)
	{
		$officersContact->sendEmail();
	}
	// If there was an error, our class and shortcode function will take care of it later on in the script
	// After all we're currently in init, it's too early to yell at the user at this point!
}


/*--- OFFICERS DIRECTORY ADMIN ---*/

include_once(dirname(__FILE__) . '/officersdir_admin.php');

// Register our function to add our admin page
add_action('admin_menu', 'addOfficersDirPage');

// Add Officers Directory to the Toole menu in WP Admin
function addOfficersDirPage()
{
	global $officersDirPageHook; // Globalized so we can get to our contextual help
	$officersDirPageHook = add_management_page('Manage Directory of Officers', 'Officers Directory', 'edit_users', __FILE__, 'officersDirAdminPage');
}

// Depending on our needs, determine what function to use to display our admin page
function officersDirAdminPage()
{
	// Our URL has specifically asked us to look up a single officer
	if (isset($_GET['officer']))
	{
		officersDirEdit(array($_GET['officer']));
	}
	// We've gotten confirmation to delete certain officers
	else if (isset($_POST['officerDeleteConfirm']))
	{
		officersDirDeleteConfirmed();
	}
	// We need to update our officers, as an edit form was submitted
	else if (isset($_POST['officerEdit_submit']))
	{
		officersDirUpdate();
	}
	// We need to add some new officers thanks to the submitted form
	else if (isset($_POST['officerAdd_submit']))
	{
		officersDirInsert();
	}
	// We've gotten some kind of form submitted, but we need to determine what we were asked to do
	else if (isset($_POST['officerAction']))
	{
		switch ($_POST['officerAction'])
		{
			case 'Add':
				officersDirAdd($_POST['newOfficerCount']);
			break;
			
			case 'Edit':
				officersDirEdit($_POST['officersEdit']);
			break;
			
			case 'Delete':
				officersDirDeleteAsk($_POST['officersEdit']);
			break;
			
			case 'Re-Order':
				officersDirReorder();
			break;
			
			case 'Add Position Type':
				officersDirAddType($_POST['newPositionType']);
			break;
			
			case 'Re-Name':
				officersDirUpdateType($_POST['updateType']);
			break;
			
			// Anything else? We don't know what's going on, so let's just go to the list
			default:
				officersDirList();
			break;
		}
	}
	// We have been asked to move some position types
	else if (isset($_GET['type']) && isset($_GET['move']))
	{
		officersDirReorderTypes($_GET['type'], $_GET['move']);
	}
	// We have been asked to delete a position type, but not confirmed yet
	else if (isset($_GET['type']) && isset($_GET['delete']))
	{
		officersDirDeleteTypeAsk($_GET['type']);
	}
	// We have confirmed that we want to delete a position type
	else if (isset($_POST['delTypeConfirm']))
	{
		// Find out if we want to do with the officers in this type
		switch ($_POST['delAction'])
		{
			case 'deleteall':
				officersDirDeleteTypeConfirmed($_POST['delTypeID']);
			break;
			
			case 'transfer':
				officersDirDeleteTypeConfirmed($_POST['delTypeID'], $_POST['delTypeTransfer']);
			break;
			
			// No radio button was selected, so go back
			default:
				officersDirDeleteTypeAsk($_POST['delTypeID'], 'Please confirm whether the officers listed under this position type should be deleted or transferred to another type.');
			break;
		}
	}
	// Nothing requested for, so let's just go to the list
	else
	{
		officersDirList();
	}
}

/*--- CONTEXTUAL HELP FOR THE OFFICERS ADMINISTRATION ---*/

function officersDirAdminHelp($contextual_help, $screen_id, $screen)
{
	global $officersDirPageHook;
	if ($screen_id == $officersDirPageHook)
	{
		include_once(dirname(__FILE__) . '/officersdir_help.php');
		
		// Our contextual help is going to depend on what sort of page we're loading
		// So we'll use the same if/else and switch statements from officersDirAdminPage
		// Unless, of course, that screen was going to take us to process a form
		if (isset($_GET['officer']))
		{
			$contextual_help = officersDirHelp_Edit();
		}
		else if (isset($_POST['officerAction']))
		{
			switch ($_POST['officerAction'])
			{
				case 'Add':
					$contextual_help = officersDirHelp_Add();
				break;
				
				case 'Edit':
					$contextual_help = officersDirHelp_Edit();
				break;
				
				case 'Delete':
					$contextual_help = officersDirHelp_Delete();
				break;
				
				// Anything else? We don't know what's going on, so let's just go to the list
				default:
					$contextual_help = officersDirHelp_List();
				break;
			}
		}
		else if (isset($_GET['type']) && isset($_GET['delete']))
		{
			$contextual_help = officersDirHelp_DeleteType();
		}
		// Nothing requested for, so let's just go to the list
		else
		{
			$contextual_help = officersDirHelp_List();
		}
		
		return $contextual_help;
	}
	return $contextual_help;
}

add_action('contextual_help', 'officersDirAdminHelp', 10, 3);
